@@ -3,3 +3,7 @@ box::use(./a[...])
 double = function (x) c(x, x)
 
 indirect_counter = function () get_counter()
+
+.hidden = 'hidden'
+
+box::export()
